// Example 3
// We will investigate choices of the number and location of knots

// Setting up
clear all
set scheme fpsaus_c
cd ${stpm3course}/data

// Load Data
use breast_EW50, clear

// keep only least and most deprived groups
keep if inlist(dep,1,5)

// stset data (restrict follow-up to 5 years)
stset survtime, failure(dead=1) exit(time 5)

// Kaplan-Meier plot
sts graph, by(dep) risktable name(km, replace)
sts graph, hazard by(dep) kernel(epan2) name(haz, replace)

// loop over a few different df
forvalues df = 1/6 {
  stpm3 i.dep, scale(lncumhazard) df(`df')
  estimates store df`df'
  predict h`df', hazard timevar(0 5, n(101)) frame(haz, mergecreate) ///
                at1(dep 1)
  predict S`df', surv timevar(0 5, n(101)) frame(surv, mergecreate) ///
                at1(dep 1)  
}

// make a legend
global order
forvalues df = 1/6 {
  estimates restore df`df'
  local aic: di %9.3f `e(AIC)'
  local bic: di %9.3f `e(BIC)'
  global order ${order} `df' "df`df': AIC = `aic': BIC = `bic'"
}


// Agreement between hazard functions
frame haz: line h? tt, sort                           ///
                       legend(order(${order}) pos(7)) ///
                       ylabel(0(0.02)0.1)

// Agreement between survival functions
//   -- Given the hazard functions, how well will the
//      survival functions agree?
frame surv: line S? tt, sort                           ///
                        legend(order(${order}) pos(7)) ///
                        ylabel(0.4(0.2)1)

//----------------------------------------------
// What happens if we have many (up to 100) df?
//   -- Will estimates become unstable?
//   -- Will models fit?
//   -- Will it affect the hazard ratio
//   -- Will it affect the standard errors?
//----------------------------------------------

// loop over different df
estimates drop df*
frame drop haz surv
foreach df in 3 4 5 10 20 50 100 {
  stpm3 i.dep, scale(lncumhazard) df(`df')
  estimates store df`df'
  predict h`df', hazard timevar(0 5, n(101)) frame(haz, mergecreate) ///
                at1(dep 1)
  predict S`df', surv timevar(0 5, n(101)) frame(surv, mergecreate) ///
                at1(dep 1)

}

// How good agreement will we get with  3, 4 or 5 df
frame haz: line h3 h4 h5 tt, sort ylabel(0(0.02)0.1)


// How about when we add 10 and 20 df?
frame haz: line h3 h4 h5 h10 h20 tt, sort ylabel(0(0.02)0.1)

// How about when we add 50 and 100 df?
frame haz: line h3 h4 h5 h10 h20 h50 h100 tt, sort ylabel(0(0.02)0.1)

// What about the hazard ratio - will they be similar
// How will the additional parameters affect the SE
estimate table df*, se(%5.4f) b(%5.4f) keep(5.dep) 

// What about the survival function with 3, 4 5 df?
frame surv: line S3 S4 S5 tt, sort

// What about the survival function with 5, 10 & 20 df?
frame surv: line  S5 S10 S20 tt, sort

// What about the survival function with 10 20, 50, 100 df?
frame surv: line S5 S10 S20 S50 S100 tt, sort


///////////////////////////////////////////////////////////////////////
//  What about knot locations?                                       //
//    -- We can specify our own knot locations.                      //
//    -- We will use the percentiles option to give knot locations   //
//       at percentiles of the distribution of event times.          //
///////////////////////////////////////////////////////////////////////

// This is equivalent to df(5)
// Note the boundary knots are at the min/max event times by default.
stpm3 i.dep, scale(lncumhazard) knots(20 40 60 80, percentile)

// Program to generate random knot positions.
program define randknots, rclass
  syntax , n(integer)
  mata: knots = sort(runiform(strtoreal(st_local("n")),1,3,97),1)
  mata: st_local("randknots",invtokens(strofreal(knots',"%3.1f")))
  return local randknots `randknots'
end

// Let's run the program 3 times.
randknots , n(4)
di "`r(randknots)'"
randknots , n(4)
di "`r(randknots)'"
randknots , n(4)
di "`r(randknots)'"

// -----------------------------------------------------------
// We will fit 10 different models with random knot locations
//   -- We store the model.
//   -- We predict the baseline hazard and survival functions.
// ------------------------------------------------------------
set seed 12415
forvalues m = 1/10 {
  randknots, n(4)
  global knots`m' `r(randknots)'
  stpm3 i.dep, scale(lncumhazard) knots(${knots`m'}, percentile)
  estimates store model`m'
  predict h`m', hazard timevar(0 5, n(101)) frame(hazk, mergecreate) ///
                at1(dep 1)
  predict S`m', surv timevar(0 5, n(101)) frame(survk, mergecreate) ///
                at1(dep 1)  
}


// make legend
global order
forvalues m = 1/10 {
  global order ${order} `m' "${knots`m'}"
}

// Look at the hazard predictions
frame hazk: twoway (line h* tt), ///
            legend(order(${order}))
            
// Look at the survival predictions
// How similar/different will they be?
frame survk: twoway (line S* tt), ///
            legend(order(${order}))         
            
// Compare log hazard ratios
// How similar/different will they be?
etable , estimates(model*) keep(5.dep) mstat(_hide) 
            