// This program produces the knot sensitivity graphs
// Paul Lambert 2024-09-07

// Load Data
clear all
set scheme fpsaus_c
global dofile stpm3_sensitivity_to_knots.do

use breast_EW50 
stset survtime, failure(dead==1) exit(time 5)

// loop over different df
capture frame drop hpred
capture frame drop spred
forvalues i = 1/10 {
  stpm3 i.dep, scale(lncumhazard) df(`i') 
  estimates store df`i'
  predict h`i', hazard zeros timevar(0 5, step(0.025)) per(1000) frame(hpred, mergecreate)
  predict S`i', surv zeros timevar(0 5, step(0.025)) frame(spred, mergecreate)
}


// produce legend (including AIC/BIC)
forvalues i = 1/10 {
  estimates restore df`i'
  local AIC`i' : display %6.1f `e(AIC)'
  local BIC`i' : display %6.1f `e(BIC)'
  local legtext `"`legtext' `i' "`i' df: AIC=`AIC`i'', BIC=`BIC`i''""'
}


// hazard graph
frame hpred {
  twoway (line h* tt), ///
         xtitle("Years from diagnosis") ///
         ytitle("Mortality rate (per 1000 pys)") ///
         legend(order(`legtext') size(*0.8) pos(6) bmargin(0 0 0 0)) ///
         ylabel(0(20)100) ///
         note("Graph code in ${dofile}", size(vsmall) span color(gs3%50))        
}

// survival graph
frame spred {
  twoway (line S* tt), ///
         xtitle("Years from diagnosis") ///
         ytitle("Survival function") ///
         legend(order(`legtext') size(*0.8) pos(7) bmargin(0 0 0 0)) ///
         ylabel(0.5(0.1)1, format(%3.1f)) ///
         note("Graph code in ${dofile}", size(vsmall) span color(gs3%50))           
}

// compare hazard ratios
estimates table df1 df3 df5 df7 df9, keep(2.dep 3.dep 4.dep 5.dep) eform se

// Placement of knots
set seed 21345
capture frame drop hknots
capture frame drop sknots
local legtext

// select internal knots randomly
forvalues i = 1/10 {
  local klist
  forvalues j = 1/4 {
    local k`j' = runiform(1,99)
    local k`j': display %3.1f `k`j''
    local klist `klist' `k`j''
  }
  numlist "`klist'", sort
  local klist `r(numlist)'
  di "`klist'"
  stpm3 i.dep, scale(lncumhazard) knots(`klist', percentile)
  predict h`i', hazard zeros timevar(0 5, step(0.025)) per(1000) frame(hknots, mergecreate)
  predict S`i', surv zeros timevar(0 5, step(0.025)) frame(sknots, mergecreate)
  local legtext `"`legtext' `i' "`klist'""'
}

// hazard graph
frame hknots {
  twoway (line h* tt), ///
         xtitle("Years from diagnosis") ///
         ytitle("Mortality rate (per 1000 pys)") ///
         legend(order(`legtext') size(*0.8) pos(6) bmargin(0 0 0 0)) ///
         ylabel(0(20)100) ///
         note("Graph code in ${dofile}", size(vsmall) span color(gs3%50))         
}

// survival graph
frame sknots {
  twoway (line S* tt), ///
         xtitle("Years from diagnosis") ///
         ytitle("Survival function") ///
         legend(order(`legtext') size(*0.8) pos(7) bmargin(0 0 0 0)) ///
         ylabel(0.5(0.1)1, format(%3.1f)) ///
         note("Graph code in ${dofile}", size(vsmall) span color(gs3%50))          
}

